module.exports = function getSchemaFactory() {
  const getScopeTree = () => {
    const scopes = Array.from(document.querySelectorAll('[itemscope]'));

    const values = scopes.map(scope => ({
      scope,
      values: [],
      parentScope: scope.parentNode.closest('[itemscope]'),
      type: scope.getAttribute('itemtype'),
    }));

    values.forEach((current, idx, allScopes) => {
      const parent = current.parentScope
        && allScopes.find(x => x.scope === current.parentScope);
      if (parent) {
        if (parent.child) {
          throw new Error('parent already has a child', parent, current);
        }
        parent.child = current;
        Object.assign(current, { parent });
      } else {
        Object.assign(current, { parent: undefined });
      }
    });

    return values;
  };

  const removeSchemaUrl = str => str.replace(/https?:\/\/schema.org\//, '');

  const getSchema = () => {
    const scopes = getScopeTree();
    const itemprops = document.querySelectorAll('[itemprop]:not([itemscope]');
    const loose = [];

    Array.from(itemprops).forEach((el) => {
      const parentScope = el.closest('[itemscope]');
      const scope = scopes.find(x => x.scope === parentScope);

      if (!parentScope) {
        const desc = {
          name: el.getAttribute('itemprop'),
          value: el.getAttribute('content') || el.innerText,
          el,
        };
        if (el.tagName === 'A') {
          desc.value = el.getAttribute('href');
        }
        loose.push(desc);
        return;
      }

      const desc = {
        scope: parentScope
          .getAttribute('itemtype')
          .replace(/https?:\/\/schema.org\//, ''),
        name: el.getAttribute('itemprop'),
        value: el.getAttribute('content') || el.innerText,
        el,
      };
      if (desc.scope === 'offer' && desc.name === 'availability') {
        desc.value = el
          .getAttribute('href')
          .replace(/https?:\/\/schema.org\//, '');
      }

      scope.values.push(desc);
    });

    // stock
    // Discontinued
    // InStock
    // InStoreOnly
    // LimitedAvailability
    // OnlineOnly
    // OutOfStock
    // PreOrder
    // PreSale
    // SoldOut

    // grab the roots
    const buildValuesObject = (scope, obj) => {
      const type = removeSchemaUrl(scope.type);
      Object.assign(
        obj,
        {
          [type]: scope.values.reduce((accum, value) => {
            Object.assign(accum, { [value.name]: value.value });
            return accum;
          }, {}),
        },
      );
      if (scope.child) {
        buildValuesObject(scope.child, obj[type]);
      }
    };

    if (loose.length) {
      console.warn('Some itemprops found without a scope', loose);
    }

    return scopes.filter(x => x.parent === undefined).reduce((accum, scope) => {
      buildValuesObject(scope, accum);
      return accum;
    }, {});
  };
  return getSchema;
};
